import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import Tether from 'tether';
import throttle from 'lodash.throttle';

import isInViewport from '../../utils/isInViewport';


const FoundationTether = React.createClass({

    propTypes: {
        children: PropTypes.node,
        target: PropTypes.func,
        tetherTo: PropTypes.string,
        width: PropTypes.string,
    },

    getDefaultProps() {
        return {
            children: null,
            target: () => {},
            tetherTo: 'content',
            width: null,
        };
    },

    componentWillMount() {
        this.tetherElement = null;
    },

    componentDidMount() {
        const element = document.getElementById(this.props.tetherTo);
        // this is mainly to support testing, should probably warn in browser if condition is met.
        if (element === null) {
            return;
        }

        this.tetherElement = document.createElement('div');
        this.tetherElement.className = 'foundation-tether';

        element.appendChild(this.tetherElement);

        this.renderChildren();

        this.tether = new Tether(this.getOptions());
        this.setOrientation();

        this.onResize = throttle(this.setOrientation, 250);
        window.addEventListener('resize', this.onResize);

        this.setWidth();
    },

    componentDidUpdate() {
        this.renderChildren();
        this.setWidth();
    },

    componentWillUnmount() {
        if (!this.tetherElement) {
            return;
        }

        window.removeEventListener('resize', this.onResize);

        this.tether.destroy();

        ReactDOM.unmountComponentAtNode(this.tetherElement);

        this.tetherElement.parentNode.removeChild(this.tetherElement);
        this.tetherElement = null;

        this.child = null;
    },

    getOptions(overrides = {}) {
        return {
            element: this.tetherElement,
            target: this.props.target(),
            attachment: 'top left',
            targetAttachment: 'bottom left',
            ...overrides,
        };
    },

    setOrientation() {
        // try right orientation.
        this.tether.setOptions(this.getOptions({
            attachment: 'top left',
            targetAttachment: 'bottom left',
        }));

        const node = ReactDOM.findDOMNode(this.child);
        if (!isInViewport(node)) {
            // try left orientation.
            this.tether.setOptions(this.getOptions({
                attachment: 'top right',
                targetAttachment: 'bottom right',
            }));

            if (!isInViewport(node)) {
                // try middle orientation.
                this.tether.setOptions(this.getOptions({
                    attachment: 'top middle',
                    targetAttachment: 'bottom middle',
                }));
            }
        }
    },

    setWidth() {
        if (this.tetherElement !== null && this.props.width !== null) {
            this.tetherElement.style.width = this.props.width;
        }
    },

    renderChildren() {
        if (!this.tetherElement) {
            return;
        }

        this.child = ReactDOM.render(this.props.children, this.tetherElement);
    },

    render() {
        return (<div></div>);
    },

});

export default FoundationTether;
