import React, { PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';

import isNullOrEmpty from '../../../utils/isNullOrEmpty';


export default ({
    block,
    displayName,
} = {}) => {

    invariant(block !== undefined, 'Missing purebem block');
    invariant(!isNullOrEmpty(displayName), 'Missing display name');

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            checked: PropTypes.bool,
            content: PropTypes.node,
            disabled: PropTypes.bool,
            onChange: PropTypes.func,
            value: PropTypes.string,
        },

        getDefaultProps() {
            return {
                checked: false,
                content: null,
                disabled: false,
                onChange: () => {},
                value: '',
            };
        },

        onClick() {
            const { checked, disabled, onChange, value } = this.props;
            if (checked || disabled) {
                return;
            }

            onChange(value);
        },

        render() {
            const { checked, content, disabled } = this.props;
            return (
                <div
                    className={ block('option-custom', { disabled, checked, unchecked: !checked }) }
                    onClick={ this.onClick }>
                    <i className={ block('option-custom-symbol') } />
                    <div className={ block('option-custom-content') }>{ content }</div>
                </div>
            );
        },

    });

};
