import React, { PropTypes } from 'react';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import invariant from 'invariant';

import * as lang from '../../utils/lang';
import isNullOrEmpty from '../../utils/isNullOrEmpty';
import reactInterpolate from '../../utils/reactInterpolate';


export default ({
    block,
    displayName,
} = {}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    const Input = React.createClass({

        displayName: `${displayName}Input`,

        mixins: [PureRenderMixin],

        propTypes: {
            disabled: PropTypes.bool,
            height: PropTypes.number,
            maxLength: PropTypes.number,
            name: PropTypes.string,
            onChange: PropTypes.func,
            placeholder: PropTypes.string,
            value: PropTypes.string,
        },

        focus() {
            this.component.focus();
            this.component.select();
        },

        render() {
            const { disabled, height, maxLength, name, onChange, placeholder, value } = this.props;
            return (
                <textarea
                    className={ block('input') }
                    disabled={ disabled }
                    maxLength={ maxLength }
                    name={ name }
                    onChange={ onChange }
                    placeholder={ placeholder }
                    ref={ (c) => this.component = c }
                    style={ { height } }
                    value={ value } />
            );
        },
    });

    return React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        propTypes: {
            disabled: PropTypes.bool,
            error: PropTypes.bool,
            focus: PropTypes.bool,
            height: PropTypes.number,
            maxLength: PropTypes.oneOfType([
                PropTypes.number,
                PropTypes.string,
            ]),
            name: PropTypes.string,
            onChange: PropTypes.func,
            placeholder: PropTypes.string,
            value: PropTypes.string,
        },

        getDefaultProps() {
            return {
                disabled: false,
                error: false,
                focus: false,
                height: null,
                maxLength: null,
                name: null,
                onChange: (/* value */) => {},
                placeholder: null,
                value: '',
            };
        },

        componentDidMount() {
            if (this.props.focus) {
                this.component.focus();
            }
        },

        onChange(evt) {
            this.props.onChange(evt.target.value);
        },

        renderRemainingCharacters() {
            const { maxLength, value } = this.props;
            if (maxLength === null) {
                return null;
            }

            const remaining = maxLength - value.length;
            return (
                <div className={ block('remaining-characters') }>
                    { reactInterpolate(lang.pgettext('foundation-remaining-characters', 'Label--characters-remaining[%(remaining)s]'), { remaining }) }
                </div>
            );
        },

        render() {
            const { disabled, error, height, maxLength, name, placeholder, value } = this.props;
            return (
                <div className={ block({ disabled, error }) }>
                    <Input
                        disabled={ disabled }
                        height={ height }
                        maxLength={ maxLength }
                        name={ name }
                        onChange={ this.onChange }
                        placeholder={ placeholder }
                        ref={ (c) => this.component = c }
                        value={ value } />
                    { this.renderRemainingCharacters() }
                </div>
            );
        },

    });

};
