const isInvalidNumber = (maxNumberOfDots) => (inp) => (inp.match(/\./g) || []).length > maxNumberOfDots;

const isInvalidFloat = isInvalidNumber(1);
const isInvalidInteger = isInvalidNumber(0);

const isValid = (replacedInput, type, value) => {
    if (Number.isNaN(value)) {
        return false;
    }

    switch (type) {
        case 'float':
            return !isInvalidFloat(replacedInput);

        case 'integer':
            return !isInvalidInteger(replacedInput);

        case 'integer-positive-only':
            return !isInvalidInteger(replacedInput) && value >= 0;

        default:
            return true;
    }
};

const isPositiveOnly = (type) => {
    switch (type) {
        case 'integer-positive-only':
            return true;

        default:
            return false;
    }
};

export default (input, type) => {
    if (input.length === 1 && input.startsWith('-') && !isPositiveOnly(type)) {
        return { value: 0, valid: true };
    }

    const replacedInput = input.replace(',', '.');
    const value = Number(replacedInput);

    if (!isValid(replacedInput, type, value)) {
        return { value: '', valid: false };
    }

    return { value, valid: true };
};
